<?php
/**
 * Защищённый запрос к api ВКонтакте
 * @package ru.vbinc.gm.framework.VKontakte
 * @author GreyMag
 * @copyright 2009
 * @version 1.6
 */

class SecureRequest
{
	/**
	 * Сообщение об ошибке
	 */
	public $error,
	/**
	 * Номер ошибки
	 */
	$errno,
	/**
	 * Посланные поля
	 */
	$requestFields;
	 
	private $_secureCode, // защищенный ключ
	$_apiId, // идентификатор приложения
	$_method, // название метода API из общего списка функций
	$_parameters = array(), // массив передаваемых параметров
	$_v = '2.0', // версия API - необязательный параметр
	$_format = 'JSON', // формат возвращаемых данных – XML или JSON. По умолчанию XML - необязательный параметр
	$_requestUrl = 'http://api.vkontakte.ru/api.php', // url для запроса
	$_testMode = false; // тестовый режим

	// список защищёных методов
	const
	/**
	 * Метод отправляет уведомление пользователю
	 */
	SEND_NOTIFICATION = 'secure.sendNotification',
	/**
	 * Метод сохраняет строку статуса приложения
	 * для последующего вывода в общем списке приложений на странице пользоваетеля
	 */
	SAVE_APP_STATUS = 'secure.saveAppStatus',
	/**
	 * Метод возвращает платежный баланс приложения
	 */
	GET_APP_BALANCE = 'secure.getAppBalance',
	/**
	 * Метод возвращает баланс пользователя на счету приложения
	 */
	GET_BALANCE = 'secure.getBalance',
	/**
	 * Метод списывает голоса со счета пользователя на счет приложения
	 */
	WITHDRAW_VOTES = 'secure.withdrawVotes',
	/**
	 * Метод возвращает историю транзакций внутри приложения
	 */
	GET_TRANSACTIONS_HISTORY = 'secure.getTransactionsHistory',
	/**
	 * Таймаут между запросами ( не болше трех в секунду )
	 */
	KONTAKT_REQUEST_DELAY = .3;


	public function __construct( $apiId, $testMode = false )
	{
		$this->_apiId = $apiId;
		$this->_secureCode = SECURE_CODE;

		$this->_parameters['api_id'] = $this->_apiId;
		$this->_parameters['v'] = $this->_v;
		$this->_parameters['format'] = $this->_format;

		if( $testMode )
		{
			$this->_testMode = true;
			$this->_parameters['test_mode'] = '1';
		}
	}

	/**
	 * SecureRequest::request()
	 * Отправка запроса на ВКонтакте
	 * @param mixed $method Название метода
	 * @param array $parameters Массив передаваемых параметров со значениями
	 * @return Объект ответа с ВКонтакте
	 */
	public function request( $method, $parameters = array() )
	{
		$this->_parameters['method'] = $method;

		if( is_array( $parameters ) )
		$this->_parameters = array_merge( $this->_parameters, $parameters );

		$this->_parameters['timestamp'] = $this->getTimestamp();
		$this->_parameters['random'] = $this->getRandom();

		$queryString = $this->createQueryString();

		$curl = curl_init();

		curl_setopt( $curl, CURLOPT_URL, $this->_requestUrl );
		curl_setopt( $curl, CURLOPT_POST, 1);
		curl_setopt( $curl, CURLOPT_POSTFIELDS, $queryString );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER , 1);

		$result = curl_exec( $curl );
		$error = curl_getinfo( $curl );

		curl_close( $curl );
				
		if ( $error['http_code'] != "200" )
		{
			$this->errno = '0';
			$this->error = 'Ошибка соединения';
			return false;
		}
		else
		{									
			$answer = json_decode( $result );//simplexml_load_string( $result );

			//if( $answer->getName() != 'response' )
			if( !isset( $answer->response ) )
			{
				//$this->error = '->'.$result . '| (' . $queryString . ')';
                //return false;
				if( isset( $answer->error ) )
				{
					$this->errno = $answer->error->error_code;
					$this->error = $answer->error->error_msg;
					$this->requestFields = $answer->error->request_params;
				}
				else $this->error = 'Неопознанная ошибка.';

				return false;
			}
			else return $answer->response;
		}
		/*{
		 header( 'Content-type: text/xml' );
		 echo '<?xml version="1.0" encoding="utf-8"?>
		 <xml>';
		 echo '<![CDATA[Запрос: ' . $queryString . ']]>';
		 echo substr( $result, 39 ) ;
		 echo '</xml>';
		 }*/

		//header( 'Location: ' . $this->_requestUrl . '?' . $queryString );
	}

	// формируем строку запроса
	private function createQueryString()
	{
		$string = '';

		foreach( $this->_parameters as $name=>$value ) $string .= $name . '=' . urlencode( $value ) . '&';

		$string .= 'sig=' . $this->createSIG();

		return $string;
	}

	// создаем подпись, в целях безопасности
	private function createSIG()
	{
		$string = '';

		ksort( $this->_parameters, SORT_STRING );

		foreach( $this->_parameters as $name=>$value ) $string .= $name . '=' . $value;

		$string .= $this->_secureCode;

		return md5( $string );
	}

	// любое случайное число для обеспечения уникальности запроса
	private function getRandom(  $length = 'rand', $max = 6, $min = 12 )
	{
		if( $length == 'rand' ) $length = rand( $max,$min );

		$symb = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
		$str = '';
		for( $i = 0; $i < $length; $i++ ) $str .= substr( $symb, rand( 0, strlen( $symb ) ), 1 );

		return $str;
	}

	// текущее значение unixtime на сервере
	private function getTimestamp()
	{
		return date( 'U' );
	}
}
?>