<?php
/**
 * @package ru.vbinc.gm.framework.VKontakte
 * @author GreyMag
 * @copyright 2009
 * @version 1.2
 */

class VKService extends Service
{
  /**
   * класс для запросов
   * @var SecureRequest
   */
  protected $_kontakt;
  /**
   * информация ( флешварс )
   * @var FlashVars
   */
  protected $_info;
  /**
   * Вызванный метод
   * @var String
   */
  protected $_method;
  /**
   * Методы, вызывать которые можно без авторизации
   * @var array
   */
  protected $_allowMethods = array();

  function __construct( $createDBC = true )
  {
    $this->_kontakt = new SecureRequest( API_ID ); // создаем класс для запросов

    parent::__construct( $createDBC );
  }

  /**
   * AuthService::beforeFilter()
   * Метод, который вызывается перед вызовом любого метода
   * @param String $calledFunc
   * @return
   */
  public function beforeFilter( $calledFunc )
  {
    $this->_method = $calledFunc;
    $this->_info = VKGlobals::getFlashVars();
    
    return ( (boolean)$this->_info || in_array( $calledFunc, $this->_allowMethods ) );
  }

  protected function setFlashvars( $flashvars )
  {
    $this->_info = new FlashVars( $flashvars );

    // если не установлено или ключ не совпадает - нафик
    if( Globals::isDebugMode() ) {
      $this->_info->isAppUser = true;
      $this->_info->apiId = defined( 'API_ID' ) ? API_ID : $this->_info->apiId;
      $this->_info->authKey = $this->getAuthKey();
    }
    
    if( !$this->_info->isAppUser ) {
      $this->error( 'Приложение не установлено' );
      return false;
    } elseif( defined( 'API_ID' ) && $this->_info->apiId != API_ID ) {
      $this->error( 'Неверное приложение' );
      return false;
    } elseif( $this->_info->authKey !== $this->getAuthKey() ) {
      $this->error( 'Ошибка безопасности' );
      return false;
    }

    
    VKGlobals::setFlashVars( $this->_info );

    return true;
  }

  // получаем код авторизации
  protected function getAuthKey()
  {
    return md5( $this->_info->apiId . '_' . $this->_info->viewerId . '_' . SECURE_CODE );
  }

  /**
   * Kontakt::transferVotes()
   * Перевод голосов между пользователем и приложением
   * @param float $votes количество голосов,
   * если отрицательное, то перевод идет с приложения к пользователю,
   * если положительное - наоборот
   * @param integer $userId - id пользователя, по умолчанию - текущий
   * @return boolean
   */
  protected function transferVotes( $votes, $userId = 0 )
  {
    if( $votes == 0 ) return true;
    elseif( $votes < 0 ) return false;

    $userId = ( $userId == 0 ) ? $this->_info->viewerId : (int)$userId;

    $votes = (int)( abs( $votes ) * 100 );

    if( $result = $this->_kontakt->request( SecureRequest::WITHDRAW_VOTES, array( 'uid' => $userId, 'votes' => $votes ) ) )
    {
      if( (int)$result == $votes ) return true;
      else {
        $this->error( 'Ошибка перевода голосов' );
        return false;
      }
    }
    else {
      $this->error( ( $this->_kontakt->errno == 502 ) ? 'У Вас на счету нет голосов' : $this->_kontakt->errno . ': ' . $this->_kontakt->error );
      return false;
    }
  }

  /**
   * RenderService::sendNotification()
   * Отправка уведомления пользователям
   * @param mixed $toUids id пользователя кому отправлять
   * или массив id'ов пользователей
   * @param string $message текст сообщения
   * @return
   */
  protected function sendNotification( $toUids, $message )
  {
    //$this->_callMethod = __METHOD__;
    //$toUid = (int)$toUid;
    $toUids = ( is_array( $toUids ) ) ? $toUids : array( (int)$toUids );

    $message = str_replace( '"', '&#34;', $message );
    $message = str_replace( "'", '&#39;', $message );
    //$message = $this->_db->escape_string( $message );

    $__arrayOfArrays = array_chunk( $toUids, 100 );

    foreach( $__arrayOfArrays as $__uidsArray )
    {
      $__uids = implode( ',', $__uidsArray );

      if( !$request = $this->_kontakt->request( SecureRequest::SEND_NOTIFICATION, array( 'uids' => $__uids, 'message' => $message ) ) )
      {
        //$this->error( $this->_kontakt->error );
        // не надо запарывать все изи-за дурацких вконтактовских глюков
        $this->error = $this->_kontakt->error;
        return false;
      }

      usleep( SecureRequest::KONTAKT_REQUEST_DELAY * 1000000 ); // а то контакт ругаться будет
      // под виндой usleep не работает
      //sleep(1); // а sleep не может задержаться меньше чем на секунду
    }

    return true;
  }
}
?>