<?php
/**
 * Защищённый сервис - требует сначала авторизации
 * @package ru.vbinc.gm.framework.amfphp
 * @author GreyMag <greymag@gmail.com>
 * @version 0.1
 */
class SecureService extends Service
{
	/**
	 * Имя сессии
	 * @var string
	 */
	const AUTH_SESSION = 'authInfo';

	/**
	 * Авторизован сейчас или нет
	 * @var bool
	 */
	protected $_isLogin = false;
	
	/**
	 * Информация авторизации
	 * @var AuthInfo
	 */
	protected $_authInfo;

	/**
	 * Методы, вызывать которые можно без авторизации
	 * @var array
	 */
	protected $_allowMethods = array();
	
	function __construct()
	{
		parent::__construct();
		array_push( $this->_allowMethods, 'auth' );
	}

	/**
	 * Авторизация
	 * @param string $pass пароль
	 * @param string $login логин
	 */
	public function auth( $pass, $login = '' )
	{
		$this->_success = $this->checkAuth( $this->_db->escape_string( $login ),
		$this->_db->escape_string( $pass ) );
		$session = Session::getInstance();
		if( $this->_success ) {
			//$_SESSION[self::AUTH_SESSION] = serialize( new AuthInfo( $login ) );
			$session->set( self::AUTH_SESSION, serialize( new AuthInfo( $login ) ) );
		} else {
			//unset( $_SESSION[self::AUTH_SESSION] );
			$session->unsetVar( self::AUTH_SESSION );
		}

		return $this->answer();
	}

	/**
	 * Проверить авторизацию
	 * Это метод надо переопределять в наследниках
	 * @param string $login
	 * @param string $pass
	 * @return bool
	 */
	protected function checkAuth( $login, $pass )
	{
		return false;
	}

	/**
	 * Метод, который вызывается перед вызовом любого метода
	 * @param String $calledFunc
	 * @return bool
	 */
	public function beforeFilter( $calledFunc )
	{
		$session = Session::getInstance();

		$this->_authInfo = unserialize( $session->get( self::AUTH_SESSION ) );

		return ( (boolean) $this->_authInfo || in_array( $calledFunc, $this->_allowMethods ) );
	}

}
?>