<?php
/**
 * DBConnect
 * Класс работы с базой данных
 * @package ru.vbinc.gm.framework.db
 * @author GreyMag <greymag@gmail.com>
 * @copyright 2009
 * @version 0.2
 * @access public
 */
class DBConnect extends mysqli
{
	/**
	 * Префикс таблиц, использующихся в текущем проекте
	 */
	public $prefix;

	/**
	 * Вместо парметров, передаваемых в контруктор,
	 * можно определить в рамках проекта соответствующие констранты:
	 * MYSQL_SERVER, MYSQL_USER, MYSQL_PASS, DB_NAME, PREFIX
	 * @param string $host хост баз данных
	 * @param string $username имя пользователя баз данных
	 * @param passwd $passwd пароль
	 * @param string $dbname имя базы данных
	 * @param string $prefix префикс таблиц, использующихся в текущем проекте
	 */
	function __construct( $host = '', $username = '', $passwd = '', $dbname = '', $prefix = '' )
	{
		if( empty( $host     ) && defined( 'MYSQL_SERVER' ) ) $host = MYSQL_SERVER;
		if( empty( $username ) && defined( 'MYSQL_USER'   ) ) $username = MYSQL_USER;
		if( empty( $passwd   ) && defined( 'MYSQL_PASS'   ) ) $passwd = MYSQL_PASS;
		if( empty( $dbname   ) && defined( 'DB_NAME'      ) ) $dbname = DB_NAME;
		if( empty( $prefix   ) && defined( 'PREFIX'       ) ) $this->prefix = PREFIX;
		else $this->prefix = $prefix;

		parent::__construct( $host, $username, $passwd, $dbname );
		$this->set_charset( "utf8" );
	}

	/**
	 * Возвращает массив ассоциативных массивов из подготовленного select запроса
	 * если строка одна - то сразу ассоциативный массив
	 * @param mysqli_stmt $prepare
	 * @return array 
	 */
	public function fetchAssocPrepare( mysqli_stmt $prepare )
	{
		$meta = $prepare->result_metadata();
		$row = array();
		while( $field = $meta->fetch_field() )
		{
			$params[] = &$row[$field->name];
		}

		call_user_func_array( array( $prepare, 'bind_result' ), $params );

		$c = array();
		$result = array();
		while( $prepare->fetch() )
		{
			foreach( $row as $key => $val )
			{
				$c[$key] = $val;
			}
			$result[] = $c;
		}
		$count = count( $result );

		switch( $count )
		{
			case 0 : return false;
			case 1 : return $result[0];
			default : return $result;
		}
	}

	/**
	 * Выполняет запрос, подставляя вместо %s имя таблицы, добавляя к нему префикс
	 * warning: %s должно встречаться в запросе не более 1 раза, метод в разработке
	 * @param string $query строка запроса
	 * @param string $table имя таблицы без префикса
	 * @return mysqli_result
	 */
	public function queryt( $query, $table )
	{
		return $this->checkReturn( $this->query( sprintf( $query, $this->prefix . $table ) ) );
	}

	/**
	 * Готовит запрос, подставляя вместо %s имя таблицы с префиксом
	 * warning: %s должно встречаться в запросе не более 1 раза, метод в разработке
     * @param string $query строка запроса
	 * @param string $table имя таблицы без префикса
	 * @return mysqli_stmt
	 */
	public function preparet( $query, $table )
	{
		return $this->checkReturn( $this->prepare( sprintf( $query, $this->prefix . $table ) ) );
	}

	/**
	 * Делает запрос для апдейта заданных полей
	 * @param array $fields4Update поле => значение
	 * @return mysqli_result
	 */
	public function updateQueryt( $table, $fields4Update, $where = '' )
	{
		$update = '';
		foreach( $fields4Update as $var => $value ) {
			if( $update != '' ) $update .= ', ';
			$update .= "`" . $var . "` = '" . $this->escape_string( $value ) . "'";
		}

		$sql = "update `%s` ";
		if( $update != '' )
		$sql .= "set " . $update;
		if( $where != '' )
		$sql .= " where " . $where;

		return $this->queryt( $sql, $table );
	}

	/**
	 * Делает запрос вставки заданных полей
	 * @param string $table таблица
	 * @param array $fields4Insert поле => значение
     * @return mysqli_result
	 */
	public function insertQueryt( $table, $fields4Insert )
	{
		/*if( count( $fields4Insert ) == 0 ) return false;
		  
		$fields = '';
		$values = '';
		foreach( $fields4Insert as $var => $value ) {
		if( $fields != '' ) {
		$fields .= ', ';
		$values .= ', ';
		}
		$fields .= "`" . $var . "`";
		$values .= "'" . $this->escape_string( $value ) . "'";
		}

		$sql = "insert into `%s` (" . $fields . ") values (" . $values . ")";
		return $this->queryt( $sql, $table );*/
		return $this->multiInsertQueryt( $table, array( $fields4Insert ) );
	}

	/**
	 * Делает запрос вставки нескольких записей с заданными полями заданных полей
	 * @param string $table таблица
	 * @param array $fields4Insert массив массивов поле => значение
     * @return mysqli_result
	 */
	public function multiInsertQueryt( $table, $fields4InsertArray )
	{
		if( count( $fields4InsertArray ) == 0 ) return false;
		 
		$fields = '';
		$valuesTotal = '';

		for( $i = 0; $i < count( $fields4InsertArray ); $i++ ) {
			if( count( $fields4InsertArray[$i] ) == 0 ) {
				//continue;
				// нехер всякую фигню сувать мне
				return false;
			}
			 
			$values = '';
			foreach( $fields4InsertArray[$i] as $var => $value ) {
				if( $values != '' ) {
					if( $i == 0 ) $fields .= ', ';
					$values .= ', ';
				}

				if( $i == 0 ) $fields .= "`" . $var . "`";
				$values .= "'" . $this->escape_string( $value ) . "'";
			}
			 
			if( $valuesTotal != '' ) $valuesTotal .= ', ';
			$valuesTotal .= '(' . $values . ')';
		}

		$sql = "insert into `%s` (" . $fields . ") values " . $valuesTotal;
		return $this->queryt( $sql, $table );
	}
	
	/**
     * @see mysqli::escape_string()
	 */
	public function escape_string( $value )
	{
		$value = stripcslashes( $value );
		return parent::escape_string( $value );
	}

	/**
	 * @see mysqli::multi_query()
	 */
	public function multi_query( $query )
	{
		if( !parent::multi_query( $query ) ) return false;
		// это мы избавляемся от ошибки
		// Commands out of sync; you can't run this command now
		// при выполнении следующего запроса после multi_query
		while( $this->next_result() );
		return true;
	}
    
	/**
	 * Проверяет либой из результатов, 
	 * чтобы он не был равен false или null,
	 * а также не являлся пустым массивом
	 * @param mixed $smt
	 * @return bool
	 */
	protected function checkReturn( $smt )
	{
		if( !$smt ) return $this->error();
		return $smt;
	}
    
	/**
	 * Обработка ошибки при выполнении запроса
	 * @return bool
	 */
	protected function error()
	{
		//throw new DBException( $this );
		return false;
	}
}

?>