<?
/**
 * Набросок валидатора стандартных полей
 * @package ru.vbinc.gm.framework.string
 * @author GreyMag
 * @version 0.1
 */
class Validation
{
	
	/**
	 * Проверка e-mail на валидность
	 * Warning! Не учитывает возможность создания email с русскими быквами!
	 * @param string $email проверяемый адрес
	 * @return string
	 */
	public static function checkEmail( $email )
	{
		return ( preg_match( "/^[-a-zA-Z0-9_\.]+@[-a-zA-Z0-9_\.]+\.[-a-zA-Z0-9_\.]+$/", $email ) > 0 ) ? true : false;
	}

	public static function checkUrl( $url )
	{
		return ( !preg_match( "/^(http\:\/\/([-a-zA-Z0-9_\.])+)\.(([-a-zA-Z0-9\/_\.])+)$/", $url ) > 0 ) ? true : false;
	}

	/**
	 * Проверка строки
	 * - должна начинаться с буквы или цифры
	 * - может содержать буквы анг и рус и цифры
	 * @param unknown_type $str
	 * @param unknown_type $maxLength
	 * @param unknown_type $minLength
	 */
	public static function checkStr( $str, $maxLength, $minLength = 1, $spaces = false, $hyphen = false, $addExpr = '' )
	{
		$str = trim( $str );
		$baseExpr = "a-zA-Z0-9а-рА-Рс-яС-ЯёЁ";
		$expr = $baseExpr;
		if( $spaces ) $expr .= " ";
		if( $hyphen ) $expr .= "\-";
		$expr .= $addExpr;

		$minLength = max( $minLength, 1 );
		//return ( !preg_match( "/^(([a-zA-Z0-9]){1}([".$expr."]){" . ( $minLength - 1 ) . ", " . ( $maxLength - 1 ) . "})$/", $str ) > 0 ) ? true : false;
		 
		return ( preg_match( "/^(([" . $baseExpr . "]){1}([" . $expr . "]){" . ( $minLength - 1 ) . "," . ( $maxLength - 1 ) . "})$/u", $str ) > 0 ) ? true : false;

	}

	/**
	 * Проверка пароля
	 * - по-умолчанию может содержать буквы анг и цифры
	 * @param unknown_type $str
	 * @param unknown_type $maxLength
	 * @param unknown_type $minLength
	 */
	public static function checkPass( $str, $maxLength, $minLength = 1 )
	{
		$str = trim( $str );
		$baseExpr = "a-zA-Z0-9";
		$expr = $baseExpr;
		//if( $spaces ) $expr .= " ";
		//if( $hyphen ) $expr .= "\-";
		// $expr .= $addExpr;

		$minLength = max( $minLength, 1 );
		//return ( !preg_match( "/^(([a-zA-Z0-9]){1}([".$expr."]){" . ( $minLength - 1 ) . ", " . ( $maxLength - 1 ) . "})$/", $str ) > 0 ) ? true : false;

		return ( preg_match( "/^(([" . $expr . "]){" . $minLength . "," . $maxLength . "})$/u", $str ) > 0 ) ? true : false;

	}
}
?>