<?php
/**
 * Утилита для записи логов - синглтон
 * @package ru.vbinc.gm.framework.utils.log
 * @author GreyMag <greymag@gmail.com>
 * @copyright 2011
 * @version 0.1
 * 
 * @property-read string $logsPath Путь для записи логов
 */
class GMLog
{
	/**
	 * Инстанция класса
	 * @var GMLog
	 */
	protected static $_instance;
    
    /**
     * Возвращает инстанцию класса. 
     * Если инстанция не создана, то создаёт
     * @return GMLog
     */
	public static function getInstance()
    {
        if( !self::$_instance ) {
            self::$_instance = new GMLog();
        }
        
    	return self::$_instance;
    }
    
    /* (non-PHPdoc)
     * @see GMLog#writeLog($var,$comment='undefined',$object=null)
     */
    public static function writeLog( $var, $comment = 'undefined', $object = null )
    {
    	self::getInstance()->addLogMessage( $var, $comment, $object );
    }  
    
    /**
     * Максимальный размер основного файла логов
     * @var int
     */
    protected $_logFileMaxSize;
    /**
     * Максимальный размер одного файла архива логов
     * @var int
     */
    protected $_logFileArchiveSize;
    /**
     * Путь по умолчанию до директории логов
     * @var string
     */
    protected $_logsDefaultPath;
    /**
     * Логгирование работает только в дебаг режиме или всегда
     * @var bool
     */
    protected $_workInDebug;
    
    /**
     * Массив логов <code>ArrayList of GMLogMessage</code>
     * @var ArrayList
     */
	protected $_logs;
	
    /**
     * Если инстанция класса уже инициализорована - 
     * конструктор породит исключение 
     */
    function __construct()
    {
    	if( self::$_instance ) throw new Exception( 'GMLog is singleton' );
    	
    	$this->_logs = new ArrayList();
    	$this->init();
    }   
    
    function __get( $var )
    {
    	switch( $var ) {
    		case 'logsPath' : return $this->_logsDefaultPath;
    	}
    }

    /**
     * Инициализировать основные параметры записи логов
     * @param string $logsPath путь до директории логов, по умолчанию ./logs/
     * @param int $logFileMaxSize максимальный размер основного файла логов, по умолчанию 100 Кб
     * @param int $logFileArchiveSize максимальный размер одного файла архива логов, по умолчанию 10 Мб
     * @param bool $workInDebug если параметр выставлен в true - логгирование работает только в режиме дебага
     */
    public function init( $logsPath = './logs/', $logFileMaxSize = 102400, $logFileArchiveSize = 10485760, $workInDebug = true )
    {
    	$this->_logsDefaultPath    = $logsPath;
    	$this->_logFileMaxSize     = $logFileMaxSize;
    	$this->_logFileArchiveSize = $logFileArchiveSize;
    }
    
    /**
     * Запись в лог
     * @param mixed $var то что будет записано в лог
     * @param string $comment строка комментария к логгируемому объекту
     * @param object $object логгирующий объект
     */
    public function addLogMessage( $var, $comment = '', $object = null )
    {
    	if( !Globals::isDebugMode() && $this->_workInDebug ) return;
    	$class = ( $object != null ) ? get_class( $object ) : 'undefined';
    	$this->_logs->push( new GMLogMessage( print_r( $var, true ), $comment, $class ) );
    }
    
 /**
     * Сохранить запомненные логи в файл, без очистки массивов
     * @param string $logFileName имя файла логов
     * @param string $logFileExt расширение файла логов, по умолчанию log 
     * @param string $logFilePath путь до директории логов, 
     * если не указан - используется путь по умолчанию
     * @return void
     */
    public function saveLog( $logFileName, $logFileExt = 'log', $logFilePath = '' )
    {
        if( $this->_logs->size() == 0 || !Globals::isDebugMode() && $this->_workInDebug ) return;
        
        if( $logFilePath == '' ) $logFilePath = $this->_logsDefaultPath;
        if( substr( $logFilePath, -1, 1 ) != '/' ) $logFilePath .= '/';
        
        // проверяем - если такой директории нет,
        // то пытаемся создать
        if( !file_exists( $logFilePath ) || !is_dir( $logFilePath ) ) {
            if( !@mkdir( $logFilePath, 0777, true ) ) return;
        }
        
        $objectLogFile = $logFilePath . $logFileName . '.' . $logFileExt;

        $log = '';
        if( file_exists( $objectLogFile ) ) {           
            // если файл уже слишком большой
            // то начинаем писать в новый
            if( filesize( $objectLogFile ) < $this->_logFileMaxSize ) $log = file_get_contents( $objectLogFile );
                                
            //{
                //copy( $objectLogFile, LOGS_PATH . $fileName . DateTimeUtils::date( 'YmdHis', time() + TIMEADJUST ) . '.' . $fileExt );
            //} else $log = file_get_contents( $objectLogFile );
        }
        
        $archive = '';        
        $log = "==========================================\n\r\n\r" . $log;
        
        foreach( $this->_logs->getArray() as /* @var $logMessage GMLogMessage */ $logMessage )
        {
            $message = $logMessage->getLogString(); 
            $log = $message . "\n\r" . $log;
            $archive .= $message . "\n\r";
        }
        
        @file_put_contents( $objectLogFile, $log );
        
        // сохраняем архив
        $i = 0;
        do {
            $i++;
            $archiveFile = $logFilePath . $logFileName . '.' . $i . '.' . $logFileExt;
        } while( file_exists( $archiveFile ) );
        
        // можно еще дописать в предыдущий - или новый создавать
        if( $i > 1 ) {
            $prevFile = $logFilePath . $logFileName . '.' . ( $i - 1 ) . '.' . $logFileExt;
            if( filesize( $prevFile ) + strlen( $archive ) <= $this->_logFileArchiveSize ) {
                $archiveFile = $prevFile;
            }
        }
        
        if( !$achiveFile = @fopen( $archiveFile, 'a' ) ) return;
        
        fwrite( $achiveFile, $archive );
        fclose( $achiveFile );
    }
    
    /**
     * Сбросить запомненные логи в файл 
     * @param string $logFileName имя файла логов
     * @param string $logFileExt расширение файла логов, по умолчанию log 
     * @param string $logFilePath путь до директории логов, 
     * если не указан - используется путь по умолчанию
     * @return void
     */
    public function flushLog( $logFileName, $logFileExt = 'log', $logFilePath = '' )
    {
    	if( $this->_logs->size() == 0 || !Globals::isDebugMode() && $this->_workInDebug ) return;
    	
    	$this->saveLog( $logFileName, $logFileExt, $logFilePath );
        $this->_logs->reset(); 
    }
} 
?>